﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;
using System.Web;
using Newtonsoft.Json;
using System.Dynamic;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking ACL API endpoints.
    /// </summary>
    public class AclClient : CelloHttpClient
    {
        public AclClient(string baseAddress = "https://localhost/api/acl/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }
        /// <summary>
        ///This method is used to invoke <c>'validate_user'</c> endpoint.
        /// </summary>
        /// <param name="companyCode">company_code</param>
        /// <param name="username">username</param>
        /// <param name="password">password</param>
        /// <returns><c>True</c>, if the user is valid, <c>false</c> otherwise</returns>
        public bool ValidateUser(string companyCode, string username, string password)
        {
            var uri = "validate_user";

            var response = this.Client.PostAsJsonAsync(uri, new LogOnDTO
            {
                company_code = companyCode,
                username = username,
                password = password
            }).Result;

            //// passing null value
            //LogOnDTO details = null;
            //var response = this.Client.PostAsJsonAsync(uri, details).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<bool>().Result;
                return res;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'login'</c> endpoint.
        /// </summary>
        /// <returns>Token</returns>
        public string Login()
        {
            var uri = "login";

            var response = this.Client.PostAsJsonAsync<object>(uri, null).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<string>().Result;
                return res;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'check_privileges'</c> endpoint.
        /// </summary>
        /// <param name="privilegeIds">array of privilege identifiers</param>
        /// <returns>dictionary of privilege with corresponding permission</returns>
        public Dictionary<string, bool> CheckPrivileges(string[] privilegeIds)
        {
            var uri = "check_privileges";

            var response = this.Client.PostAsJsonAsync(uri, privilegeIds).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<Dictionary<string, bool>>().Result;
                return res;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_user_roles'</c> endpoint.
        /// </summary>
        /// <param name="userId">user identifier</param>
        /// <param name="tenantId">tenant identifier</param>
        /// <returns>collection of user roles</returns>
        public IEnumerable<string> GetUserRoles(Guid userId, Guid? tenantId = null)
        {
            var uri = "get_user_roles?user_id=" + userId.ToString();

            //// test-with out user Id.
            //var uri = "get_user_roles?user_id=";

            if (tenantId.HasValue)
            {
                uri += "&tenant_id=" + tenantId.Value;
            }

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var users = response.Content.ReadAsAsync<string[]>().Result;
                return users;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_user_privileges'</c> endpoint.
        /// </summary>
        /// <param name="userId">user identifier</param>
        /// <param name="tenantId">tenant identifier</param>
        /// <returns>collection of user privileges</returns>
        public IEnumerable<string> GetUserPrivileges(Guid userId, Guid? tenantId = null)
        {
            var uri = "get_user_privileges?user_id=" + userId.ToString();

            ////test-with out user Id.
            //var uri = "get_user_privileges?user_id=";

            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var users = response.Content.ReadAsAsync<string[]>().Result;
                return users;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'check_tenant_access_privilege'</c> endpoint.
        /// </summary>
        /// <param name="userId">user identifier</param>
        /// <param name="privilege">privilege</param>
        /// <param name="tenantId">tenant identifier</param>
        /// <returns><c>True</c>, if the given user has the privilege against the given tenant is valid, <c>false</c> otherwise</returns>
        public bool CheckTenantAccessPrivilege(Guid userId, string privilege, Guid tenantId)
        {
            var uri = "check_tenant_access_privilege?user_id=" + userId.ToString();
            uri += "&privilege=" + HttpUtility.UrlEncode(privilege);
            uri += "&tenant_id=" + tenantId.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<bool>().Result;
                return res;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'check_access_for_resource'</c> endpoint.
        /// </summary>
        /// <param name="path">UI path</param>
        /// <param name="resourceType">either URI or BusinessLogic</param>
        /// <returns><c>True</c>, if the given URI has the privilege against the given tenant is valid, <c>false</c> otherwise</returns>
        public bool CheckUrlAccess(string path, string resourceType)
        {
            var uri = "check_access_for_resource?resource=" + @path;
            uri += "&resource_type=" + resourceType;
            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<string>().Result;
                return Convert.ToBoolean(res);
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenant_role_details'</c> endpoint.
        /// </summary>
        /// <param name="tenantId">tenant identifier</param>      
        /// <returns>collection role details</returns>
        public List<RoleDTO> GetTenantRoleDetails(Guid tenantId)
        {
            var uri = "get_tenant_role_details?tenant_id=" + tenantId;

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<List<RoleDTO>>().Result;
                return res;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_all_tenant_role_details'</c> endpoint.
        /// </summary>
        /// <param name="tenantId">tenant identifier</param>      
        /// <returns>collection of all tenant role details</returns>
        public List<RoleDTO> GetAllTenantRoleDetails(Guid tenantId)
        {
            var uri = "get_all_tenant_role_details?tenant_id=" + tenantId;

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<List<RoleDTO>>().Result;
                return res;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_global_role_details'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>      
        /// <returns>collection of global role details</returns>
        public List<RoleDTO> GetGlobalRoleDetails()
        {
            var uri = "get_global_role_details";

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<List<RoleDTO>>().Result;
                return res;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_all_global_role_details'</c> endpoint.
        /// </summary>
        /// <param name="tenant_id">tenant identifier</param>      
        /// <returns>collection of all global role details</returns>
        public List<RoleDTO> GetAllGlobalRoleDetails()
        {
            var uri = "get_all_global_role_details";

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<List<RoleDTO>>().Result;
                return res;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_user_roles_for_tenant'</c> endpoint.
        /// </summary>
        /// <param name="userId">user identifier</param>
        /// <param name="stratifiedTenantId">stratified tenant identifier</param>
        /// <param name="roleAssigningTenantId">role assigning tenant identifier</param>
        /// <returns>collection of user roles for tenant</returns>
        public IEnumerable<string> GetUserRolesForTenant(Guid userId, Guid stratifiedTenantId, Guid roleAssigningTenantId)
        {
            var uri = "get_user_roles_for_tenant?user_id=" + userId;
            uri += "&stratified_tenant_id=" + stratifiedTenantId;
            uri += "&role_assigning_tenant_id=" + roleAssigningTenantId;

            var response = this.Client.GetAsync(uri).Result;
            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<string[]>().Result;
                return result;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_global_roles_by_tenant'</c> endpoint.
        /// </summary>
        /// <param name="tenantId">tenant identifier</param>
        /// <returns>collection of global role details for tenant.</returns>
        public IEnumerable<RoleDTO> GetGlobalRolesForTenant(Guid tenantId)
        {
            var uri = "get_global_roles_by_tenant?tenant_id=" + tenantId;
            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<List<RoleDTO>>().Result;
                return result;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'delete_user_roles'</c> endpoint.
        /// </summary>
        /// <param name="userId">user identifier</param>
        /// <param name="roleIds">role identifier</param>
        /// <param name="tenantId">tenant identifier</param>
        public void DeleteUserRoles(Guid userId, string[] roleIds, Guid tenantId)
        {
            UserRolesDTO userRoleDetails = new UserRolesDTO();
            userRoleDetails.user_id = userId;
            userRoleDetails.role_ids = roleIds;
            userRoleDetails.tenant_id = tenantId;

            var uri = "delete_user_roles";
            var response = this.Client.PostAsJsonAsync(uri, userRoleDetails).Result;

            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'put_user_to_roles'</c> endpoint.
        /// </summary>
        /// <param name="userRoles">UserRolesDTO model</param>
        public void AddUserToRoles(UserRolesDTO userRoles)
        {
            var uri = "put_user_to_roles";
            var response = this.Client.PutAsJsonAsync(uri, userRoles).Result;

            if (!response.IsSuccessStatusCode)
            {
                // error in updating user
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'put_users_to_role'</c> endpoint.
        /// </summary>
        /// <param name="userRoles">UserRolesDTO model</param>
        public void AddUsersToRole(UserRolesDTO userRoles)
        {
            var uri = "put_users_to_role";
            var response = this.Client.PutAsJsonAsync(uri, userRoles).Result;

            if (!response.IsSuccessStatusCode)
            {
                // error in updating user
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'post_update_user_roles_implicit'</c> endpoint.
        /// </summary>
        /// <param name="userRoles">UserRolesDTO model</param>
        public void ImplictUpdateUserRoles(UserRolesDTO userRoles)
        {
            var uri = "post_update_user_roles_implicit";
            var response = this.Client.PostAsJsonAsync(uri, userRoles).Result;

            if (!response.IsSuccessStatusCode)
            {
                // error in updating user
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'post_update_user_roles_explicit'</c> endpoint.
        /// </summary>
        /// <param name="userRoles">UserRolesDTO model</param>
        public void ExplicitUpdateUserRoles(UpdateUserRolesDTO userRoles)
        {
            var uri = "post_update_user_roles_explicit";
            var response = this.Client.PostAsJsonAsync(uri, userRoles).Result;

            if (!response.IsSuccessStatusCode)
            {
                // error in updating user
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'apply_datascope'</c> endpoint.
        /// </summary>
        /// <param name="entityId">entity identifier</param>
        /// <param name="referenceIds">array of reference identifiers</param>
        /// <param name="dataviewId">data view identifiers</param>
        /// <returns>JSON formatted string</returns>
        public string ApplyDatascope(string entityId, string[] referenceIds, string dataviewId)
        {
            var uri = "apply_datascope";

            var response = this.Client.PostAsJsonAsync(uri, new
            {
                entity_id = entityId,
                reference_ids = referenceIds,
                dataview_id = dataviewId
            }).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<string>().Result;
                return res;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

    }
}
