﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using Newtonsoft.Json;
using System.Net.Http;
using System.Dynamic;
using System.Collections;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking Tenants API endpoints.
    /// </summary>
    public class TenantsClient : CelloHttpClient
    {
        public TenantsClient(string baseAddress = "https://localhost/api/tenants/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }

        /// <summary>
        /// This method is used to invoke <c>'search_tenants'</c> endpoint.
        /// </summary>
        /// <param name="condition">tenant search condition details.</param>
        /// <returns>tenant search result</returns>
        public TenantSearchResultDTO SearchTenants(CelloSaaS.RestApiSDK.TenantSearchConditionDTO condition)
        {
            var uri = "search_tenants";

            var response = this.Client.PostAsJsonAsync(uri, condition).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<TenantSearchResultDTO>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenant'</c> endpoint.
        /// </summary>
        /// <param name="tenantId">tenant identifier.</param>
        /// <returns>specified tenant details</returns>
        public TenantDTO GetTenant(Guid tenantId)
        {
            var uri = "get_tenant?tenant_id=" + tenantId.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<TenantDTO>().Result;
                return entity;
            }

            if (response.StatusCode == System.Net.HttpStatusCode.NotFound)
            {
                return null;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenant_by_url'</c> endpoint.
        /// </summary>
        /// <param name="loginURL">login url details.</param>
        /// <returns>tenant details</returns>
        public TenantDTO GetTenantByUrl(Uri loginURL)
        {
            var uri = "get_tenant_by_url?login_url=" + HttpUtility.UrlEncode(loginURL.ToString());

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<TenantDTO>().Result;
                return entity;
            }

            if (response.StatusCode == System.Net.HttpStatusCode.NotFound)
            {
                return null;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenant_info'</c> endpoint.
        /// </summary>
        /// <returns>collection of tenant details.</returns>
        public IEnumerable<TenantDTO> GetTenantInfo()
        {
            var uri = "get_tenant_info";
            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<TenantDTO[]>().Result;
                return result;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        ///  This method is used to invoke <c>'get_stratified_tenant_by_user'</c> endpoint.
        /// </summary>
        /// <param name="userId">user unique identifier</param>
        /// <param name="tenantId">tenant unique identifier</param>
        /// <returns>collection of stratified tenant details.</returns>
        public Dictionary<string, TenantDTO> GetStratfiedTenantByUser(Guid userId, Guid tenantId)
        {
            var uri = "get_stratified_tenant_by_user?user_id=" + userId;
            uri += "&tenant_id=" + tenantId;

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<Dictionary<string, TenantDTO>>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenant_details_by_friendly_code'</c> endpoint.
        /// </summary>
        /// <param name="friendlyCode">code string of the tenant.</param>
        /// <returns>tenant details.</returns>
        public TenantDTO GetTenantDetailsByFriendlyCode(string friendlyCode)
        {
            var uri = "get_tenant_details_by_friendly_code?friendly_code=" + friendlyCode;

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<TenantDTO>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenantId_from_friendly_code'</c> endpoint.
        /// </summary>
        /// <param name="friendlyCode">code string of the tenant.</param>
        /// <returns>tenant identifier (GUID)</returns>
        public Guid GetTenantIdFromFriendlyCode(string friendlyCode)
        {
            var uri = "get_tenantId_from_friendly_code?friendly_code=" + friendlyCode;
            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                return response.Content.ReadAsAsync<Guid>().Result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_tenants_in_Trail'</c> endpoint.
        /// </summary>
        /// <returns>collection of tenant details.</returns>
        public Dictionary<string, TenantDTO> GetTenantsInTrail()
        {
            var uri = "get_tenants_in_Trail";

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var entity = response.Content.ReadAsAsync<Dictionary<string, TenantDTO>>().Result;
                return entity;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        ///  This Client used to invoke the delete_user_roles_and_tenant_mapping api method.
        /// </summary>
        /// <param name="userId">The user unique identifier</param>
        /// <param name="roleId">The role identifier</param>
        /// <param name="tenantId">The tenant unique identifier</param>
        public void DeleteUserRolesandTenantMapping(Guid userId, string roleId, Guid tenantId)
        {
            var uri = "delete_user_roles_and_tenant_mapping?user_id=" + userId;
            uri += "&role_id=" + roleId;
            uri += "&tenant_id=" + tenantId;
            var response = this.Client.DeleteAsync(uri).Result;

            if (response.StatusCode != System.Net.HttpStatusCode.NoContent)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }       
      
    }
}
