﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using Newtonsoft.Json;
using System.Net.Http;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This client is used for invoking Users API endpoints.
    /// </summary>
    public class UserDetailsClient : CelloHttpClient
    {
        public UserDetailsClient(string baseAddress = "https://localhost/api/users/", string username = "", string api_key = "")
            : base(baseAddress, username, api_key)
        {
        }

        /// <summary>
        /// This method is used to invoke <c>'get_all_users'</c> endpoint.
        /// </summary>
        /// <param name="tenantId">tenant identifier</param>
        /// <returns>collection of all user details<UserDetailsDTO></returns>
        public IEnumerable<UserDetailsDTO> GetAllUsers(Guid? tenantId = null)
        {
            var uri = tenantId.HasValue ? "get_all_users?tenant_id=" + tenantId.Value.ToString() : "get_all_users";

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var users = response.Content.ReadAsAsync<UserDetailsDTO[]>().Result;
                return users;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        ///  This method is used to invoke <c>'get_user'</c> endpoint.
        /// </summary>
        /// <param name="userId">user identifier</param>
        /// <param name="tenantId">tenant identifier</param>
        /// <returns>specified user details</returns>
        public UserDetailsDTO GetUserByUserId(Guid userId, Guid? tenantId = null)
        {
            var uri = "get_user?user_id=" + userId.ToString();
            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var user = response.Content.ReadAsAsync<UserDetailsDTO>().Result;
                return user;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'get_user_by_user_name'</c> endpoint.
        /// </summary>
        /// <param name="userName">user name.</param>
        /// <param name="tenantId">tenant identifier</param>
        /// <returns>specified user details</returns>
        public UserDetailsDTO GetUserByUsername(string userName, Guid? tenantId = null)
        {
            var uri = "get_user_by_user_name?user_name=" + HttpUtility.UrlEncode(userName);
            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var user = response.Content.ReadAsAsync<UserDetailsDTO>().Result;
                return user;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'activate_user'</c> endpoint..
        /// </summary>
        /// <param name="userId">user identifier</param>
        /// <param name="tenantId">tenant identifier</param>
        public void ActivateUser(Guid userId, Guid? tenantId = null)
        {
            var uri = "activate_user?user_id=" + userId.ToString();
            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();

            var response = this.Client.GetAsync(uri).Result;

            if (response.StatusCode == System.Net.HttpStatusCode.NotFound)
            {
                throw new Exception("user not found or already in active state");
            }
            else if (response.StatusCode != System.Net.HttpStatusCode.NoContent)
            {
                // error in deleting user
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'update_first_time_logon'</c> endpoint.
        /// </summary>
        /// <param name="userId">user identifier</param>
        /// <param name="password">password</param>
        /// <param name="securityQuestion">security question</param>
        /// <param name="securityAnswer">security answer</param>
        /// <param name="firstLogOn">first log on</param>
        /// <param name="tenantId">tenant identifier</param>
        public void UpdateFirstTimeLogOn(Guid userId, string password, string securityQuestion, string securityAnswer, bool firstLogOn, Guid? tenantId = null)
        {
            var uri = "update_first_time_logon?user_id=" + userId;
            uri += "&password=" + HttpUtility.UrlEncode(password);
            uri += "&security_question=" + securityQuestion;
            uri += "&security_answer=" + securityAnswer;
            uri += "&first_logOn=" + firstLogOn;
            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();

            var response = this.Client.PostAsync(uri, null).Result;
            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'validate_security_answer'</c> endpoint.
        /// </summary>
        /// <param name="friendlyCode">friendlyCode</param>
        /// <param name="userName">userName</param>
        /// <param name="answer">answer</param>
        /// <returns><c>True</c>, if the security answer is valid, <c>false</c> otherwise</returns>
        public bool ValidateSecurityAnswer(string friendlyCode, string userName, string answer)
        {
            var uri = "validate_security_answer?friendly_code=" + friendlyCode;
            uri += "&user_name=" + userName;
            uri += "&answer=" + HttpUtility.UrlEncode(answer);
            var response = this.Client.GetAsync(uri).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<bool>().Result;
                return res;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'change_password'</c> endpoint.
        /// </summary>
        /// <param name="userName">userName</param>
        /// <param name="tenantId">tenant identifier</param>
        /// <param name="oldPassword">old password</param>
        /// <param name="newPassword">new Password</param>
        public void ChangePassword(string userName, string oldPassword, string newPassword, Guid? tenantId = null)
        {
            var uri = "change_password?user_name=" + userName;

            uri += "&old_password=" + HttpUtility.UrlEncode(oldPassword);
            uri += "&new_password=" + HttpUtility.UrlEncode(newPassword);

            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();
            var response = this.Client.GetAsync(uri).Result;
            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'forgot_password'</c> endpoint.
        /// </summary>
        /// <param name="userName">user name</param>
        /// <param name="tenantId">tenant identifier</param>
        public void ForgotPassword(string userName, Guid? tenantId = null)
        {
            var uri = "forgot_password?user_name=" + userName;
            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();
            var response = this.Client.GetAsync(uri).Result;
            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        ///  This method is used to invoke <c>'update_user_password'</c> endpoint.
        /// </summary>
        /// <param name="userId">user identifier</param>
        /// <param name="password">password</param>
        /// <param name="tenantId">tenant identifier</param>
        public void UpdateUserPassword(Guid userId, string password, Guid? tenantId = null)
        {
            var uri = "update_user_password?user_id=" + userId;
            uri += "&password=" + HttpUtility.UrlEncode(password);
            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();
            var response = this.Client.PostAsync(uri, null).Result;

            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'update_password_expiry'</c> endpoint.
        /// </summary>
        public void UpdatePasswordExpiry()
        {
            var uri = "update_password_expiry";

            var response = this.Client.GetAsync(uri).Result;

            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        ///  This method is used to invoke <c>'reset_password'</c> endpoint.
        /// </summary>
        /// <param name="userName">user name</param>
        /// <param name="tenantId">tenant identifier</param>
        public void ResetPassword(string userName, Guid? tenantId = null)
        {
            var uri = "reset_password?user_name=" + HttpUtility.UrlEncode(userName.ToString());

            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();

            var response = this.Client.PostAsync(uri, null).Result;
            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        ///  This method is used to invoke <c>'reset_password'</c> endpoint.
        /// </summary>
        /// <param name="userName">user name</param>
        /// <param name="tenantId">tenant identifier</param>
        /// <param name="passwordQuestion">password question</param>
        /// <param name="passwordAnswer">password answer</param>
        public void ResetPassword(string userName, string passwordQuestion, string passwordAnswer, Guid? tenantId = null)
        {
            var uri = "reset_password?user_name=" + HttpUtility.UrlEncode(userName.ToString());
            uri += "&password_question=" + passwordQuestion;
            uri += "&password_answer=" + passwordAnswer;

            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();

            var response = this.Client.PostAsync(uri, null).Result;

            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'reset_password'</c> endpoint.
        /// </summary>
        /// <param name="userIds">array of user identifiers</param>
        /// <param name="password">password</param>
        /// <param name="tenantId">tenant identifier</param>
        public void ResetPassword(Guid[] userIds, string password, Guid tenantId)
        {
            ResetPasswordDTO resetPasswordDTO = new ResetPasswordDTO()
            {
                user_ids = userIds,
                password = password,
                tenant_id = tenantId
            };
            var uri = "reset_password?";

            var response = this.Client.PostAsJsonAsync(uri, resetPasswordDTO).Result;

            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }

        }

        /// <summary>
        /// This method is used to invoke <c>'unlock_user_accounts'</c> endpoint.
        /// </summary>
        /// <param name="userIds">user identifiers</param>
        public void UnlockUserAccounts(Guid[] userIds)
        {
            var uri = "unlock_user_accounts?";
            var response = this.Client.PostAsJsonAsync(uri, userIds).Result;

            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }

        }

        /// <summary>
        /// This method is used to invoke <c>'lock_user_accounts'</c> endpoint.
        /// </summary>
        /// <param name="userIds">user identifiers</param>
        public List<dynamic> lockUserAccounts(Guid[] userIds)
        {
            var uri = "lock_user_accounts?";

            var response = this.Client.PostAsJsonAsync(uri, userIds).Result;

            if (response.IsSuccessStatusCode)
            {
                var res = response.Content.ReadAsAsync<List<dynamic>>().Result;
                return res;
            }
            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'lock_all_user_accounts'</c> endpoint.
        /// </summary>
        /// <param name="tenantId">tenant identifier</param>
        public void LockAllUserAccounts(Guid tenantId)
        {
            var uri = "lock_all_user_accounts?tenant_id=" + HttpUtility.UrlEncode(tenantId.ToString());

            var response = this.Client.PostAsync(uri, null).Result;
            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'unlock_all_user_accounts'</c> endpoint.
        /// </summary>
        /// <param name="tenantId">tenant identifier</param>
        public void UnLockAllUserAccounts(Guid tenantId)
        {
            var uri = "unlock_all_user_accounts?tenant_id=" + HttpUtility.UrlEncode(tenantId.ToString());
            var response = this.Client.PostAsync(uri, null).Result;
            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        ///  This method is used to invoke <c>'force_user_password_reset'</c> endpoint.
        /// </summary>
        /// <param name="userIds">user identifiers</param>
        /// <returns></returns>
        public void ForceUserPasswordReset(Guid[] userIds)
        {
            var uri = "force_user_password_reset?";

            var response = this.Client.PostAsJsonAsync(uri, userIds).Result;

            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }

        }

        /// <summary>
        ///  This method is used to invoke <c>'put_user'</c> endpoint.
        /// </summary>
        /// <param name="user">required data for create an user</param>
        /// <returns>user identifier</returns>
        public Guid CreateUser(UserDetailsDTO user)
        {
            var uri = "put_user";

            var response = this.Client.PutAsJsonAsync(uri, user).Result;

            if (response.IsSuccessStatusCode)
            {
                var entityId = response.Content.ReadAsAsync<Guid>().Result;
                return entityId;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

        /// <summary>
        /// This method is used to invoke <c>'post_user'</c> endpoint.
        /// </summary>
        /// <param name="user">required data for update an user</param>
        public void UpdateUser(UserDetailsDTO user)
        {
            var uri = "post_user";
            var response = this.Client.PostAsJsonAsync(uri, user).Result;

            if (!response.IsSuccessStatusCode)
            {
                // error in updating user
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        ///  This method is used to invoke <c>'delete_user'</c> endpoint.
        /// </summary>
        /// <param name="user_id">user identifier</param>
        /// <param name="tenant_id">tenant identifier</param>
        public void DeleteUser(Guid userId, Guid? tenantId = null)
        {
            var uri = "delete_user?user_id=" + userId.ToString();
            if (tenantId.HasValue) uri += "&tenant_id=" + tenantId.Value.ToString();

            var response = this.Client.DeleteAsync(uri).Result;

            if (!response.IsSuccessStatusCode)
            {
                throw new Exception(response.Content.ReadAsStringAsync().Result);
            }
        }

        /// <summary>
        /// This method is used to invoke <c>'search_user'</c> endpoint.
        /// </summary>
        /// <param name="condition">condition</param>
        /// <returns>search result</returns>
        public CelloSaaS.RestApiSDK.UserDetailsSearchResultDTO Search(CelloSaaS.RestApiSDK.UserDetailsSearchConditionDTO condition)
        {
            var response = this.Client.PostAsJsonAsync("search_user", condition).Result;

            if (response.IsSuccessStatusCode)
            {
                var result = response.Content.ReadAsAsync<CelloSaaS.RestApiSDK.UserDetailsSearchResultDTO>().Result;
                return result;
            }

            throw new Exception(response.Content.ReadAsStringAsync().Result);
        }

    }

}
