﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CelloSaaS.RestApiSDK
{
    /// <summary>
    /// This Data Transfer Object contains the details of tenant.
    /// </summary>
    public class TenantDTO : BaseDTO
    {
        /// <summary>
        /// Unique identifier of the tenant.
        /// </summary>
        public Guid id { get; set; }

        /// <summary>
        ///  Unique identifier of the parent tenant.
        /// </summary>
        public Guid? parent_id { get; set; }

        /// <summary>
        /// Unique code string of the tenant.
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string friendly_code { get; set; }

        /// <summary>
        /// Name of the tenant.
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string name { get; set; }

        /// <summary>
        /// Description of the tenant.
        /// </summary>
        [MaxLength(1000)]
        public string description { get; set; }

        /// <summary>
        /// Tenant Types.
        /// </summary>
        [Required]
        [MaxLength(20)]
        public string type { get; set; }

        /// <summary>
        /// Website Address of the tenant.
        /// </summary>
        public Uri website { get; set; }

        /// <summary>
        /// Login URL of the tenant.
        /// </summary>
        public Uri login_url { get; set; }

        /// <summary>
        /// If true then the bills will be automatically debited.
        /// </summary>
        public bool enable_auto_debit { get; set; }

        /// <summary>
        /// If true then tenant is self registered
        /// </summary>
        public bool is_self_registered { get; set; }

        /// <summary>
        /// Unique identifier of Data partition
        /// </summary>
        public Guid? data_partition_id { get; set; }

        /// <summary>
        /// Size of the company.
        /// </summary>
        public string company_size { get; set; }

        /// <summary>
        /// Approval Status of the Tenant.
        /// </summary>
        public string approval_status { get; set; }

        /// <summary>
        /// Address details of the Tenant
        /// </summary>
        [Required]
        public AddressDTO address { get; set; }

        /// <summary>
        /// Contact details of the tenant
        /// </summary>
        [Required]
        public ContactDetailsDTO contact { get; set; }
    }

    /// <summary>
    /// This Data Transfer Object contains properties of tenant,user and license details.
    /// </summary>
    public class CreateTenantDTO
    {
        /// <summary>
        /// Tenant details
        /// </summary>
        [Required]
        public TenantDTO tenant { get; set; }

        /// <summary>
        /// User details of tenant.
        /// </summary>
        [Required]
        public UserDetailsDTO admin_user { get; set; }

        /// <summary>
        /// License details of the tenant.
        /// </summary>
        [Required]
        public TenantLicenseDTO license { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object is used update the tenant
    /// </summary>
    public class UpdateTenantDTO
    {
        /// <summary>
        /// tenant details
        /// </summary>
        public TenantDTO tenant { get; set; }
        /// <summary>
        /// license details
        /// </summary>
        public TenantLicenseDTO license { get; set; }
    }

    /// <summary>
    /// This Data Transfer Object contains the contact details of tenant.
    /// </summary>
    public class ContactDetailsDTO : BaseDTO
    {
        /// <summary>
        /// Unique identifier of the Contact
        /// </summary>
        public Guid id { get; set; }

        /// <summary>
        /// First name of tenant
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string first_name { get; set; }

        /// <summary>
        /// Last Name of the tenant
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string last_name { get; set; }

        /// <summary>
        /// Email Id of the tenant.
        /// </summary>
        [Required]
        [EmailAddress]
        [MaxLength(255)]
        public string email { get; set; }

        /// <summary>
        /// Phone no the tenant.
        /// </summary>
        [Required]
        [Phone]
        [MaxLength(255)]
        public string phone { get; set; }

        /// <summary>
        /// fax no the tenant
        /// </summary>
        [Required]
        [Phone]
        [MaxLength(255)]
        public string fax { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object is used to search the tenant
    /// </summary>
    public class TenantSearchConditionDTO
    {
        /// <summary>
        /// parent tenant identifier
        /// </summary>
        public Guid? parent_id { get; set; }
        /// <summary>
        /// array of tenant ids
        /// </summary>
        public Guid[] tenant_ids { get; set; }
        /// <summary>
        /// data partition id
        /// </summary>
        public Guid? data_partition_id { get; set; }
        /// <summary>
        /// friendly code
        /// </summary>
        public string friendly_code { get; set; }
        /// <summary>
        /// tenant name
        /// </summary>
        public string tenant_name { get; set; }
        /// <summary>
        /// first name
        /// </summary>
        public string first_name { get; set; }
        /// <summary>
        /// last name
        /// </summary>
        public string last_name { get; set; }
        /// <summary>
        /// phone
        /// </summary>
        public string phone { get; set; }
        /// <summary>
        /// email
        /// </summary>
        public string email { get; set; }
        /// <summary>
        /// website 
        /// </summary>
        public string website { get; set; }
        /// <summary>
        /// login url
        /// </summary>
        public string login_url { get; set; }
        /// <summary>
        /// tenant type
        /// </summary>
        public string tenant_type { get; set; }
        /// <summary>
        /// indicate whether the tenant is self registered or not
        /// </summary>
        public bool? is_self_registered { get; set; }
        /// <summary>
        /// indicate whether auto debit is enabled or not
        /// </summary>
        public bool? enable_auto_debit { get; set; }
        /// <summary>
        /// indicate whether tenant have payment account or not
        /// </summary>
        public bool? without_payment_account { get; set; }
        /// <summary>
        /// indicate whether the tenant have child or not
        /// </summary>
        public bool? has_child_tenants { get; set; }
        /// <summary>
        /// indicate the online users
        /// </summary>
        public bool? has_online_users { get; set; }
        /// <summary>
        /// page size
        /// </summary>
        public int page_size { get; set; }
        /// <summary>
        /// page number
        /// </summary>
        public int page_no { get; set; }
    }
    /// <summary>
    /// This Data Transfer Object is used to get the tenant search result
    /// </summary>
    public class TenantSearchResultDTO
    {
        /// <summary>
        /// collection of tenant result
        /// </summary>
        public IEnumerable<TenantDTO> result { get; set; }
        /// <summary>
        /// total count
        /// </summary>
        public int total_count { get; set; }
        /// <summary>
        /// condition
        /// </summary>
        public TenantSearchConditionDTO condition { get; set; }
    }

}
