﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Runtime.Serialization;
using System.Web;

namespace CelloSaaS.RestApiSDK
{
    public class LogOnDTO
    {
        /// <summary>
        /// Unique code string for company.
        /// </summary>
        [Required]
        public string company_code { get; set; }

        /// <summary>
        /// Unique identification of user used to log on the system..
        /// </summary>
        [Required]
        public string username { get; set; }

        /// <summary>
        /// String of character used to log on the system.
        /// </summary>
        [Required]
        public string password { get; set; }
    }

    /// <summary>
    /// This Data Transfer Object contains the  details of the user.
    /// </summary>
    [Serializable]
    [DataContract]
    public class UserDetailsDTO : BaseDTO
    {
        /// <summary>
        /// Unique identifier of the user
        /// </summary>
        [DataMember]
        public Guid id { get; set; }

        /// <summary>
        /// Unique identifier of the tenant
        /// </summary>
        [DataMember]
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// First name of the user.
        /// </summary>
        [Required]
        [MaxLength(200)]
        [DataMember]
        public string first_name { get; set; }

        /// <summary>
        /// Last Name of the user.
        /// </summary>
        [Required]
        [MaxLength(200)]
        [DataMember]
        public string last_name { get; set; }

        /// <summary>
        /// Description about user
        /// </summary>
        [MaxLength(1000)]
        [DataMember]
        public string description { get; set; }

        /// <summary>
        /// User name of the user.
        /// </summary>
        [Required]
        [MaxLength(200)]
        [DataMember]
        public string username { get; set; }

        /// <summary>
        /// password of the user.
        /// </summary>
        [MaxLength(200)]
        [DataMember]
        public string password { get; set; }

        /// <summary>
        /// Email Address of the user.
        /// </summary>
        [Required]
        [MaxLength(255)]
        [EmailAddress]
        [DataMember]
        public string email { get; set; }

        /// <summary>
        /// Date and time of the user activity.
        /// </summary>
        [DataMember]
        public DateTime last_activity { get; set; }

        /// <summary>
        /// Date and time of the user last login.
        /// </summary>
        [DataMember]
        public DateTime last_logon { get; set; }

        /// <summary>
        /// Date and time of the user was last locked out.
        /// </summary>
        [DataMember]
        public DateTime last_locked { get; set; }

        /// <summary>
        ///  Tell where user account is locked or not.
        /// </summary>
        [DataMember]
        public bool is_locked { get; set; }

        /// <summary>
        /// Tell whether user is first time login to the system
        /// </summary>
        [DataMember]
        public bool is_first_time_user { get; set; }

        /// <summary>
        /// Address details of user.
        /// </summary>
        [DataMember]
        public AddressDTO address { get; set; }
    }

    /// <summary>
    /// This Data Transfer Object contains details about the user address
    /// </summary>
    public class AddressDTO : BaseDTO
    {
        /// <summary>
        /// Unique Identifier of the address
        /// </summary>
        public Guid id { get; set; }

        /// <summary>
        /// Address of the user.
        /// </summary>
        [Required]
        [MaxLength(500)]
        public string address1 { get; set; }

        /// <summary>
        /// Secondary address of the user.
        /// </summary>
        [MaxLength(500)]
        public string address2 { get; set; }

        /// <summary>
        /// Name of the country
        /// </summary>
        [Required]
        [MaxLength(255)]
        public string country { get; set; }

        /// <summary>
        /// State
        /// </summary>
        [Required]
        [MaxLength(255)]
        public string state { get; set; }

        /// <summary>
        /// City
        /// </summary>
        [Required]
        [MaxLength(255)]
        public string city { get; set; }

        /// <summary>
        /// Zip code
        /// </summary>
        [Required]
        [MaxLength(15)]
        public string zipcode { get; set; }
    }

    /// <summary>
    /// This Data Transfer Object contains properties of the user role tenant.
    /// </summary>
    public class UserRolesDTO
    {
        /// <summary>
        /// Unique identifier of the users.
        /// </summary>
        public Guid[] user_ids { get; set; }

        /// <summary>
        /// Unique identifier of the user.
        /// </summary>
        public Guid user_id { get; set; }

        /// <summary>
        /// Unique identifier of the tenant.
        /// </summary>
        public Guid tenant_id { get; set; }

        /// <summary>
        /// Unique identifier of the tenants.
        /// </summary>
        public Guid[] tenant_ids { get; set; }

        /// <summary>
        /// Unique identifier of the role.
        /// </summary>
        public string role_id { get; set; }

        /// <summary>
        /// Unique identifier of the roles.
        /// </summary>
        public string[] role_ids { get; set; }
    }

    /// <summary>
    /// This Data Transfer Object contains the properties of user details search condition
    /// </summary>
    public class UserDetailsSearchConditionDTO
    {
        /// <summary>
        /// Tenant Unique Identifier
        /// </summary>
        public Guid? tenant_id { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public UserDetailsSearchField? search_field { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public string search_value { get; set; }

        /// <summary>
        /// No of the page
        /// </summary>
        public int page_no { get; set; }

        /// <summary>
        /// Size of the page
        /// </summary>
        public int page_size { get; set; }

        /// <summary>
        /// User details sort string
        /// </summary>
        public UserDetailsSortString? sort_string { get; set; }

        /// <summary>
        /// user details sort direction
        /// </summary>
        public SortExpression sort_direction { get; set; }

        /// <summary>
        /// status of user.
        /// </summary>
        public bool? status { get; set; }
    }

    /// <summary>
    /// UserDetailsSearchResult DTO
    /// </summary>
    public class UserDetailsSearchResultDTO
    {
        /// <summary>
        /// collection of user details.
        /// </summary>
        public IEnumerable<UserDetailsDTO> result { get; set; }

        /// <summary>
        /// total count user.
        /// </summary>
        public int total_count { get; set; }

        /// <summary>
        /// UserDetailsSearchConditionDTO
        /// </summary>
        public UserDetailsSearchConditionDTO search_condition { get; set; }

        /// <summary>
        /// search count.
        /// </summary>
        public int search_count { get; set; }
    }

    /// <summary>
    /// ResetPassword DTO
    /// </summary>
    public class ResetPasswordDTO
    {
        /// <summary>
        /// Unique identifier of the users.
        /// </summary>
        [Required]
        public Guid[] user_ids { get; set; }

        /// <summary>
        /// A string containing the password that used to logon into system.
        /// </summary>
        [Required]
        [MaxLength(200)]
        public string password { get; set; }

        /// <summary>
        /// Unique identifier of the tenant.
        /// </summary>
        [Required]
        public Guid tenant_id { get; set; }

    }

    /// <summary>
    ///   User Details Sort string constants
    /// </summary>
    public enum UserDetailsSortString
    {
        /// <summary>
        ///   Field - First Name
        /// </summary>
        first_name,

        /// <summary>
        ///   Field - User ID
        /// </summary>
        user_id,

        /// <summary>
        ///   Field - Last Name
        /// </summary>
        last_name,

        /// <summary>
        ///   Field - User Name
        /// </summary>
        user_name,

        /// <summary>
        ///   Field - Email ID
        /// </summary>
        email,

        /// <summary>
        ///   Field - Is user account Locked out
        /// </summary>
        is_locked,
    }

    /// <summary>
    ///  User Details Search Field DTO
    /// </summary>
    public enum UserDetailsSearchField
    {
        /// <summary>
        ///   Field - User Name
        /// </summary>
        user_name,

        /// <summary>
        ///   Field - Email ID
        /// </summary>
        email,

        /// <summary>
        ///   email or user _name
        /// </summary>
        both,
    }

    /// <summary>
    ///   Sort expression constants
    /// </summary>
    public enum SortExpression
    {
        /// <summary>
        ///   Ascending Order
        /// </summary>
        Decending,

        /// <summary>
        ///   Descending Order
        /// </summary>
        Ascending,
    }
}