﻿using System;
using System.Linq;
using System.Collections.Generic;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    /// This class is resposible for unit testing of ACL API.
    /// </summary>
    [TestClass]
    public class AclClientUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "acl/";
        public static string AclClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        public static Guid SharedTenantId = Guid.Parse("f1ec1c3c-985e-e411-bb88-000c29361821");

        public static Guid EmptyUserId = Guid.Empty;
        public static Guid EmptyTenantId = Guid.Empty;

        public static string CompanyCode = "company";
        public static string InvalidCompanyCode = "company12345";

        public static string UserName = "admin@company.com";
        public static string InvalidUserName = "administrator123@company.com";
        public static string Password = "company#123";
        public static string InvalidPassword = "QWERTY#123";

        public static string SecurityQuestion = "What is your city of birth?";
        public static string InvalidSecurityQuestion = "How is your role model";
        public static string SecurityAnswer = "NewYork";
        public static string InvalidSecurityAnswer = "Tokyo";

        public static string OldPassword = Password;
        public static string NewPassword = "company@123";

        public static string UpdatePassword = "copmany%123";
        public static string ResetPassword = "company$123";

        public static string InvalidPrivilege = "View_Systems";
        public static string GlobalRoleMember = "GR$Member";

        ExceptionHelper exceptionHelper = null;

        public AclClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }

        /// <summary>
        /// This method is used to validate the user.
        /// </summary>
        [TestMethod]
        public void ValidateUser()
        {
            using (var client = GetClient())
            {
                bool result = client.ValidateUser(CompanyCode, UserName, Password);
                Assert.IsTrue(result);
            }
        }

        /// <summary>
        /// This method is used to test user validation with invalid parameters.
        /// </summary>
        [TestMethod]
        public void ValidateUser_InvalidParameters()
        {
            using (var client = GetClient())
            {
                bool result = client.ValidateUser(InvalidCompanyCode, WrongUserId.ToString(), InvalidPassword);
                Assert.AreEqual(result, false);
            }
        }

        /// <summary>
        /// This method is used to test user validation with wrong data.
        /// </summary>
        [TestMethod]
        public void ValidateUserWithWrongData()
        {
            using (var client = GetClient())
            {
                #region Passing all value as null Value
                try
                {
                    bool result = client.ValidateUser(null, null, null);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);                    
                }
                #endregion Passing all value as null Value

                #region Passing all value as Empty Value
                try
                {
                    bool result = client.ValidateUser(string.Empty, string.Empty, string.Empty);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);  
                }
                #endregion

                #region Passing company code as null Value
                try
                {
                    bool result = client.ValidateUser(null, UserName, Password);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion

                #region Passing company code as Empty Value
                try
                {
                    bool result = client.ValidateUser(string.Empty, UserName, Password);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion

                #region Passing username as null Value
                try
                {
                    bool result = client.ValidateUser(CompanyCode, null, Password);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion

                #region Passing username  as Empty Value
                try
                {
                    bool result = client.ValidateUser(CompanyCode, string.Empty, Password);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion

                #region Passing password as null Value
                try
                {
                    bool result = client.ValidateUser(CompanyCode, UserName, null);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion

                #region Passing password  as Empty Value
                try
                {
                    bool result = client.ValidateUser(CompanyCode, UserName, string.Empty);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion
            }
        }
        /// <summary>
        /// This method is used to check the privileges.
        /// </summary>
        [TestMethod]
        public void CheckPrivileges()
        {
            using (var client = GetClient())
            {
                Dictionary<string, Boolean> result = client.CheckPrivileges(new String[] { "Add_Tenant" });
                Assert.IsNotNull(result);
            }
        }

        /// <summary>
        /// This method is used to test check privileges with null privilege identifiers
        /// </summary>
        [TestMethod]
        public void CheckPrivilage_NullPrivilage_Ids()
        {
            using (var client = GetClient())
            {
                Dictionary<string, Boolean> result = client.CheckPrivileges(null);
                Assert.IsNull(result);

                try
                {
                    result = client.CheckPrivileges(new String[] { "Add_Chart", "Add_User", null });
                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentNullErrorCode); 
                }
            }
        }
        
        /// <summary>
        /// This method is used to test check privileges with invalid privilege identifiers.
        /// </summary>
        [TestMethod]
        public void CheckPrivilage_InvalidPrivilage_Ids()
        {
            using (var client = GetClient())
            {
                Dictionary<string, Boolean> result = client.CheckPrivileges(null);
                Assert.IsNull(result);

                result = client.CheckPrivileges(new String[] { "Add_Chart1", "Add_User1" });
                Assert.IsTrue(result.Values.Contains(false));
                Assert.IsFalse(result.Values.Contains(true));              
            }
        }

        /// <summary>
        /// This method is used to test the user roles with user and tenant identifier.
        /// </summary>
        [TestMethod]
        public void TestGetUserRoles()
        {
            using (var client = GetClient())
            {

                #region Test without tenant Id

                var userRoles = client.GetUserRoles(UserId);
                Assert.IsNotNull(userRoles);

                #endregion

                #region Test with tenant Id

                var userRoles2 = client.GetUserRoles(UserId, TenantId);
                Assert.IsNotNull(userRoles2);
                #endregion

            }
        }

        /// <summary>
        /// This method is used to test the user roles for tennats with invalid identifiers.
        /// </summary>
        [TestMethod]
        public void GetUserRolesForTenant_IncorrectIds()
        {
            using (var client = GetClient())
            {
                #region Test without invalid user  Id

                var userRoles = client.GetUserRoles(WrongUserId, TenantId);
                Assert.AreEqual(0, userRoles.Count());

                #endregion

                #region Test without invalid tenant Id
                
                    userRoles = client.GetUserRoles(UserId, WrongTenantId);
                    Assert.AreEqual(0, userRoles.Count());                
              
                #endregion
            }

        }

        /// <summary>
        /// This method is used to test user role for tenant.
        /// </summary>
        [TestMethod]
        public void GetUserRolesForTenant_Other()
        {
            Guid tenantId = TenantId;
            Guid userId = UserId;
            using (var client = GetClient())
            {
                var userRoles = client.GetUserRoles(userId, tenantId);
                Assert.IsNotNull(userRoles);
            }
        }

        /// <summary>
        /// This method is used to test get privileges.
        /// </summary>
        [TestMethod]
        public void GetUserPrivileges()
        {
            using (var client = GetClient())
            {
                #region Test without tenant Id

                var userRoles = client.GetUserPrivileges(UserId);
                Assert.IsNotNull(userRoles);

                #endregion

                #region Test with tenant Id

                var userRoles2 = client.GetUserPrivileges(UserId, TenantId);
                Assert.IsNotNull(userRoles2);
                #endregion

            }
        }
        /// <summary>
        /// This method is used to test the user privileges with invalid parameters.
        /// </summary>
        [TestMethod]
        public void GetUserPrivileges_IncorrectIds()
        {
            using (var client = GetClient())
            {
                #region Test without invalid user  Id

                Guid tenantId = TenantId;
                Guid userId = WrongUserId;

                var userRoles = client.GetUserPrivileges(userId, tenantId);

                Assert.IsNull(userRoles);

                #endregion

                #region Test without invalid tenant Id

                    Guid tenantId1 = WrongTenantId;
                    Guid userId1 = UserId;

                    userRoles = client.GetUserPrivileges(userId1, tenantId1);
                    Assert.IsNull(userRoles);
               
                #endregion
            }

        }

        /// <summary>
        /// This method is used to test the user privileges with wrong user identifier.
        /// </summary>
        [TestMethod]
        public void GetUserPrivileges_WrongUserID()
        {
                using (var client = GetClient())
                {
                    var userPrivileges = client.GetUserPrivileges(WrongUserId);
                    Assert.IsNull(userPrivileges);
                }
           
        }

        /// <summary>
        /// This method is used to check the tenant access privileges.
        /// </summary>
        [TestMethod]
        public void CheckTenantAccessPrivilege()
        {

            using (var client = GetClient())
            {
                #region test current tenant

                bool access = client.CheckTenantAccessPrivilege(UserId, "Add_Chart", TenantId);
                Assert.AreEqual(access, true);

                #endregion
            }
        }

        /// <summary>
        /// This method is used to test the check tenant access privileges with invalid user identifier. 
        /// </summary>
        [TestMethod]
        public void InValidCheckTenantAccessPrivilege()
        {
            using (var client = GetClient())
            {
                #region pass empty tenant Id and user id
                try
                {

                    bool access1 = client.CheckTenantAccessPrivilege(EmptyUserId, "Add_Chart", EmptyUserId);
                    Assert.AreEqual(access1, true);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode); 
                }
                #endregion

                #region test -invalid tenant Id and user id-

                bool access2 = client.CheckTenantAccessPrivilege(WrongUserId, "Add_Chart", WrongTenantId);
                Assert.AreEqual(access2, false);

                #endregion

                #region test -invalid tenant Id


                bool access3 = client.CheckTenantAccessPrivilege(UserId, "Add_Chart", WrongTenantId);
                Assert.AreEqual(access3, false);

                #endregion

                #region test -invalid user Id

                bool access4 = client.CheckTenantAccessPrivilege(WrongUserId, "Add_Chart", TenantId);
                Assert.AreEqual(access3, false);

                #endregion
            }
        }

        /// <summary>
        /// This method is used to test URL access.
        /// </summary>
        [TestMethod]
        public void Test_Correct_URLAccess()
        {
            using (var client = GetClient())
            {
                    bool result = client.CheckUrlAccess("/settingstemplate/index", "0");
                    Assert.IsTrue(result);
            }
        }
        /// <summary>
        /// This method is used to test the URL access with invalid parameters
        /// </summary>
        [TestMethod]
        public void Test_InCorrect_URLAccess()
        {
            using (var client = GetClient())
            {
                try
                {   
                    bool result1 = client.CheckUrlAccess("/settingstemplate/index", "1");
                    Assert.IsFalse(result1);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentNullErrorCode); 
                }
            }
        }
        /// <summary>
        /// This method is used to test URL access with null and invalid resource type.
        /// </summary>
        [TestMethod]
        public void Test_Correct_URLAccess_With_Invalid()
        {
            using (var client = GetClient())
            {

                #region both value are null
                try
                {

                    bool result1 = client.CheckUrlAccess(null, null);
                    Assert.IsFalse(result1);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion

                #region pass the path as null
                try
                {

                    bool result1 = client.CheckUrlAccess(null, "UI");
                    Assert.IsFalse(result1);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion

                #region pass the resource type as null
                try
                {

                    bool result1 = client.CheckUrlAccess("/settingstemplate/index", null);
                    Assert.IsFalse(result1);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion

                #region pass the resource type as "Test" insetad of "UI"
                try
                {

                    bool result1 = client.CheckUrlAccess("/settingstemplate/index", "Test");
                    Assert.IsFalse(result1);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode); 
                }
                #endregion
            }
        }

        /// <summary>
        /// This method is used to test tenant role details.
        /// </summary>
        [TestMethod]
        public void GetTenantRoleDetails()
        {
            using (var client = GetClient())
            {
                #region test with login tenant

                List<RoleDTO> roleDetails = client.GetTenantRoleDetails(TenantId);
                Assert.IsNotNull(roleDetails);

                #endregion

            }
        }

        /// <summary>
        /// This method is used to test tenant role details with invalid data.
        /// </summary>
        [TestMethod]
        public void GetTenantRoleDetailsInValidData()
        {
            using (var client = GetClient())
            {
                #region test -invalid tenant Id

                try
                {
                    List<RoleDTO> roleDetails = client.GetTenantRoleDetails(WrongTenantId);
                    Assert.IsNotNull(roleDetails);
                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.UnauthorizedAccessExceptionErrorCode); 
                }
                #endregion


                #region test -pass guid.empty

                try
                {
                    List<RoleDTO> roleDetails = client.GetTenantRoleDetails(Guid.Empty);
                    Assert.IsNotNull(roleDetails);
                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode); 
                }
                #endregion
            }
        }

        /// <summary>
        /// This method is used to test all tenant roel details.
        /// </summary>
        [TestMethod]
        public void GetAllTenantRoleDetails()
        {
            using (var client = GetClient())
            {
                #region test current tenant
                List<RoleDTO> roleDetails = client.GetAllTenantRoleDetails(TenantId);
                Assert.IsNotNull(roleDetails);
                #endregion

            }
        }

        /// <summary>
        /// This method is used to test all tenant role details with invalid data.
        /// </summary>
        [TestMethod]
        public void GetAllTenantRoleDetailsInValidData()
        {
            using (var client = GetClient())
            {
                #region test -invalid tenant Id

                try
                {
                    List<RoleDTO> roleDetails = client.GetAllTenantRoleDetails(WrongTenantId);
                    Assert.IsNotNull(roleDetails);
                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.UnauthorizedAccessExceptionErrorCode); 
                }
                #endregion


                #region test -pass guid.empty

                try
                {
                    List<RoleDTO> roleDetails = client.GetAllTenantRoleDetails(Guid.Empty);
                    Assert.IsNotNull(roleDetails);
                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode); 
                }
                #endregion
            }
        }

        /// <summary>
        /// This method is used to test global role details.
        /// </summary>
        [TestMethod]
        public void GetGlobalRoleDetails()
        {
            #region test current tenant
            using (var client = GetClient())
            {
                List<RoleDTO> roleDetails = client.GetGlobalRoleDetails();
                Assert.IsNotNull(roleDetails);
            }
            #endregion
        }
        /// <summary>
        /// This method is used to test all global role details.
        /// </summary>
        [TestMethod]
        public void GetAllGlobalRoleDetails()
        {
            #region test current tenant
            using (var client = GetClient())
            {
                List<RoleDTO> roleDetails = client.GetAllGlobalRoleDetails();
                Assert.IsNotNull(roleDetails);
            }
            #endregion
        }

        /// <summary>
        /// This method is used to test user roles for tenant.
        /// </summary>
        [TestMethod]
        public void GetUserRolesForTenant()
        {
                using (var client = GetClient())
                {
                    #region Test-Login tenant Id

                    var result = client.GetUserRolesForTenant(UserId, TenantId, TenantId);
                    Assert.IsNotNull(result);

                    #endregion Test-Login tenant Id                  
                }
        }

        /// <summary>
        /// This method is used to test user role for tenant with invalid data.
        /// </summary>
        [TestMethod]
        public void GetUserRolesForTenantWithInValidData()
        {

            using (var client = GetClient())
            {
                #region Test-wrong user id
              
                    var result = client.GetUserRolesForTenant(WrongUserId, TenantId, TenantId);
                    Assert.IsNull(result);
                
                #endregion Test-wrong user id

                #region Test-wrong stratifiedtenant  id
           
                    var result1 = client.GetUserRolesForTenant(UserId, WrongTenantId, TenantId);
                    Assert.IsNull(result1);
                
                #endregion Test-wrong stratifiedtenant id

                #region Test-pass guid.empty
                try
                {
                    var result2 = client.GetUserRolesForTenant(Guid.Empty, Guid.Empty, Guid.Empty);
                    Assert.IsNull(result2);
                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode); 
                }
                #endregion Test-wrong stratifiedtenant id
            }

        }

        /// <summary>
        /// This method is used to test the global roles for tenant.
        /// </summary>
        [TestMethod]
        public void GetGetGlobalRolesForTenant()
        {

            using (var client = GetClient())
            {
                #region Test-Login tenant Id
                var result = client.GetGlobalRolesForTenant(TenantId);
                Assert.IsNotNull(result);
                #endregion Test-Login tenant Id
            }
        }

        /// <summary>
        /// This method is used to test the global roles for tenant with invalid data.
        /// </summary>
        [TestMethod]
        public void GetGetGlobalRolesForTenantWithInValidData()
        {

            using (var client = GetClient())
            {
                #region Test-wrong  tenant Id
                try
                {
                    var result = client.GetGlobalRolesForTenant(WrongTenantId);

                    Assert.IsNull(result);
                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.UnauthorizedAccessExceptionErrorCode); 
                }

                #endregion Test-wrong tenant Id

                #region Test-Empty  tenant Id
                try
                {

                    var result1 = client.GetGlobalRolesForTenant(Guid.Empty);
                    Assert.IsNull(result1);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode); 
                }
                #endregion Test-Empty tenant Id
            }
        }

        /// <summary>
        /// This method is used to test delete user roles.
        /// </summary>
        [TestMethod]
        public void DeleteUserRoles()
        {          
                using (var client = GetClient())
                {    
                    client.DeleteUserRoles(UserId, new string[] { GlobalRoleMember }, TenantId);
                }
        }

        /// <summary>
        /// This method is used to test  delete user roles with invalid data.
        /// </summary>
        [TestMethod]
        public void DeleteUserRolesWithInvalidData()
        {
            using (var client = GetClient())
            {             
                // pass Guid.Empty
                try
                {
                    client.DeleteUserRoles(Guid.Empty, new string[] { "b590cd25-3093-df11-8deb-001ec9dab123$HRRole" }, Guid.Empty);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode); 
                }

                // pass null role id
                try
                {
                    client.DeleteUserRoles(Guid.Empty, null, Guid.Empty);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode); 
                }

                // pass invalid role id
                try
                {
                    client.DeleteUserRoles(Guid.Empty, new string[] { "b590cd25-3093-df11-8deb-001ec9dab12$HRRole" }, Guid.Empty);
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode); 
                }
            }

        }
        /// <summary>
        /// This method is used to test the add user to roles.
        /// </summary>
        [TestMethod]
        public void PutUserToRoles()
        {
            using (var client = GetClient())
            {
                client.AddUserToRoles(new UserRolesDTO
                {
                    user_id = UserId,
                    role_ids = new string[] { GlobalRoleMember },
                    tenant_id = TenantId
                });
                var userRoles = client.GetUserRoles(UserId, TenantId);
                Assert.IsTrue(userRoles != null && userRoles.Contains(GlobalRoleMember));
                DeleteUserRoles();
            }
        }
        /// <summary>
        /// This method is used to test the add user to roles with invalid data.
        /// </summary>
        [TestMethod]
        public void PutUserRoles_invalid()
        {

                using (var client = GetClient())
                {
                    #region pass null role id
                    try
                    {
                        client.AddUserToRoles(new UserRolesDTO
                        {
                            user_id = UserId,
                            role_ids = null,
                            tenant_id=TenantId
                        });

                    }
                    catch (Exception Ex)
                    {
                        string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                        Assert.IsNotNull(errorCode);
                        Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                    }
                    #endregion

                    #region pass empty role id
                    try
                    {
                        client.AddUserToRoles(new UserRolesDTO
                        {
                            user_id = UserId,
                            role_ids = new string[0],
                            tenant_id=TenantId
                        });

                    }
                    catch (Exception Ex)
                    {
                        string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                        Assert.IsNotNull(errorCode);
                        Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                    }
                    #endregion

                    #region pass empty user id
                    try
                    {
                        client.AddUserToRoles(new UserRolesDTO
                        {
                            user_id = Guid.Empty,
                            role_ids = new string[0],
                            tenant_id=TenantId
                        });

                    }
                    catch (Exception Ex)
                    {
                        string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                        Assert.IsNotNull(errorCode);
                        Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                    }
                    #endregion
                }
        }
        /// <summary>
        /// This method is used to test the add user to roles with invalid user identifier.
        /// </summary>
        [TestMethod]
        public void PutUserRoles_invalid_user_id()
        {
            try
            {
                using (var client = GetClient())
                {
                    DeleteUserRoles();
                    client.AddUserToRoles(new UserRolesDTO
                    {
                        user_id = WrongTenantId,
                        role_ids = new string[] { GlobalRoleMember },
                        tenant_id=TenantId
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.RoleExceptionErrorCode);
            }
        }

        /// <summary>
        /// This method is used to test the add user to roles with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void PutUserRoles_invalid_tenant_id()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.AddUserToRoles(new UserRolesDTO
                    {
                        user_id = UserId,
                        role_ids = new string[] { GlobalRoleMember },
                        tenant_id = WrongTenantId
                    });
                }

            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.TenantExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the add users to role.
        /// </summary>
        [TestMethod]
        public void PutUsersToRole()
        {
            using (var client = GetClient())
            {
                DeleteUserRoles();
                client.AddUsersToRole(new UserRolesDTO
                {
                    user_ids = new Guid[] { UserId },
                    role_id = GlobalRoleMember,
                    tenant_id = TenantId
                });

                var userRoles = client.GetUserRoles(UserId, TenantId);
                Assert.IsTrue(userRoles != null && userRoles.Contains(GlobalRoleMember));
                DeleteUserRoles();


            }
        }
        /// <summary>
        /// This method is used to test the add users to role with duplicate role identifiers.
        /// </summary>
        [TestMethod]
        public void PutUsersToRole_DuplicateRoleIds()
        {
            using (var client = GetClient())
            {
                try
                {
                    client.AddUsersToRole(new UserRolesDTO
                    {
                        user_ids = new Guid[] { UserId },
                        role_id = GlobalRoleMember,
                        tenant_id=TenantId
                    });                  

                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                }

            }
        }
        /// <summary>
        /// This method is used to test the add users to role with invalid data.
        /// </summary>
        [TestMethod]
        public void PutUsersToRoles_invalid()
        {
            try
            {
                using (var client = GetClient())
                {
                    #region pass null role id
                    try
                    {
                        client.AddUsersToRole(new UserRolesDTO
                        {
                            user_ids = new Guid[] { UserId, Guid.Parse("252e88b7-c968-e411-b593-000c29361821") },
                            role_id = null,
                            tenant_id=TenantId
                        });

                    }
                    catch(Exception Ex)
                    {
                        string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                        Assert.IsNotNull(errorCode);
                        Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                    }
                    #endregion

                    #region pass empty user_ids
                    try
                    {
                        client.AddUsersToRole(new UserRolesDTO
                        {
                            user_ids = new Guid[0],
                            role_id = GlobalRoleMember,
                            tenant_id=TenantId
                        });

                    }
                    catch(Exception Ex)
                    {
                        string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                        Assert.IsNotNull(errorCode);
                        Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                    }
                    #endregion



                    #region pass empty role id
                    try
                    {
                        client.AddUsersToRole(new UserRolesDTO
                        {
                            user_ids = new Guid[] { UserId, Guid.Parse("252e88b7-c968-e411-b593-000c29361821") },
                            role_id = string.Empty,
                            tenant_id=TenantId
                        });

                    }
                    catch(Exception Ex)
                    {
                        string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                        Assert.IsNotNull(errorCode);
                        Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                    }
                    #endregion
                }
            }
            catch (Exception ex)
            {
                Assert.IsTrue(true);
            }
        }
        /// <summary>
        /// This method is used to test the add users to role with invalid user identifier.
        /// </summary>
        [TestMethod]
        public void PutUserToRole_invalid_user_id()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.AddUsersToRole(new UserRolesDTO
                    {
                        user_ids = new Guid[] { WrongTenantId },
                        role_id = GlobalRoleMember,
                        tenant_id=TenantId
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.RoleExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the add users to role with invalid tenant identifier.
        /// </summary>
        [TestMethod]
        public void PutUserToRole_invalid_tenant_id()
        {
            try
            {
                using (var client = GetClient())
                {
                    DeleteUserRoles();
                    client.AddUsersToRole(new UserRolesDTO
                    {
                        user_ids = new Guid[] { UserId },
                        role_id =GlobalRoleMember,
                        tenant_id = WrongTenantId
                    });
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.RoleExceptionErrorCode);
            }
        }

        /// <summary>
        /// This method is used to test the implicit update user roles.
        /// </summary>
        [TestMethod]
        public void ImplictUpdateUserRoles_With_Valid()
        {
            using (var client = GetClient())
            {
                DeleteUserRoles();
                client.ImplictUpdateUserRoles(new UserRolesDTO
                {
                    user_id = UserId,
                    tenant_id = TenantId,
                    role_ids = new string[] { GlobalRoleMember }
                });
                var userRoles = client.GetUserRoles(UserId, TenantId);
                Assert.IsTrue(userRoles != null && userRoles.Contains(GlobalRoleMember));
                DeleteUserRoles();
            }
        }

        /// <summary>
        /// This method is used to test the implicit update user roles with invalid data.
        /// </summary>
        [TestMethod]
        public void ImplictUpdateUserRoles_With_Empty()
        {
            using (var client = GetClient())
            {
                // pass user id  as empty
                try
                {
                    client.ImplictUpdateUserRoles(new UserRolesDTO
                    {
                        user_id = Guid.Empty,
                        role_ids = new string[] { "GR$Product_Admin", "GR$Tenant_Admin" },
                        tenant_id=TenantId
                    });

                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                }

                // pass role_ids as null
                try
                {
                    client.ImplictUpdateUserRoles(new UserRolesDTO
                    {
                        user_id = UserId,
                        role_ids = null,
                        tenant_id=TenantId
                    });

                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                }
            }

        }

        /// <summary>
        /// This method is used to test the implicit update user roles with invalid data.
        /// </summary>
        [TestMethod]
        public void ImplictUpdateUserRoles_With_invalid_data()
        {
            using (var client = GetClient())
            {
                // pass user id  as wrong id
                try
                {
                    client.ImplictUpdateUserRoles(new UserRolesDTO
                    {
                        user_id = WrongUserId,
                        role_ids = new string[] { "GR$Product_Admin", "GR$Tenant_Admin" },
                        tenant_id=TenantId
                    });

                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.RoleExceptionErrorCode);
                }

                // pass role_ids as null
                try
                {
                    client.ImplictUpdateUserRoles(new UserRolesDTO
                    {
                        user_id = UserId,
                        role_ids = new string[] { "GR$Product_Admin", "GR$Tenant_Admin" },
                        tenant_id = WrongTenantId
                    });
                }
                catch(Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.UnauthorizedAccessExceptionErrorCode);
                }
            }

        }
        /// <summary>
        /// Client 
        /// </summary>
        /// <returns></returns>
        private static AclClient GetClient()
        {
            var client = new AclClient(
              baseAddress: AclClientBaseAddress,
              username: ClientUserName,
              api_key: APIKey);
            return client;
        }
    }
}
