﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;

namespace WebApiSDKUnitTest
{
    /// <summary>
    /// This class is resposible for unit testing of EntityMetadatas API.
    /// </summary>
    [TestClass]
    public class EntityMetadatasClientUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "EntityMetadatas/";
        public static string EntityMetadatasClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static Guid UserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B3");
        public static Guid TenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab123");

        public static string UserDetailsFormEntityId = "UserDetailsForm";
        public static string AddressEntityId = "Address";
        public static string UserEntityId = "User";
        public static string InvalidEntityId = "User12";

        public static Guid WrongUserId = Guid.Parse("3398F837-B988-4708-999D-D3DFE11875B4");
        public static Guid WrongTenantId = Guid.Parse("b590cd25-3093-df11-8deb-001ec9dab122");

        ExceptionHelper exceptionHelper = null;

        public EntityMetadatasClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }

        #region TestGetAllEntityMetadatas

        #region positive
        /// <summary>
        /// This method is used to test the get all entity metadatas.
        /// </summary>
        [TestMethod]
        public void TestGetAllEntityMetadatas()
        {
            using (var client = GetClient())
            {
                var data = client.GetAllEntityMetadatas();
                Assert.IsNotNull(data);
            }
        }
        #endregion
        #endregion

        #region TestGetEntityMetadata

        #region positive
        /// <summary>
        /// This method is used to test the get entity metadata with entity identifier.
        /// </summary>
        [TestMethod]
        public void TestGetEntityMetadata()
        {
            using (var client = GetClient())
            {
                var data = client.GetEntityMetadata(AddressEntityId);
                Assert.IsNotNull(data);
            }
        }
        #endregion


        #region negative
        /// <summary>
        /// This method is used to test the get entity metadatas with invalid entity identifer.
        /// </summary>
        [TestMethod]
        public void TestGetEntityMetadata_Fail()
        {
            using (var client = GetClient())
            {
                try
                {
                    var data = client.GetEntityMetadata(InvalidEntityId);
                    Assert.IsNull(data);
                }
                catch (Exception Ex)
                {
                    Assert.IsNotNull(Ex.Message);
                    Assert.IsTrue(Ex.Message.Contains(ExceptionHelper.InvaldiEntityIDErrorMsg));
                }
            }
        }

        /// <summary>
        /// This method is used to test the get entity metadata with empty entity identifier.
        /// </summary>
        [TestMethod]
        public void TestGetEntityMetadata_EmptyEntityID()
        {
            try
            {
                using (var client = GetClient())
                {
                    var data = client.GetEntityMetadata("");
                    Assert.IsNull(data);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
            }
        }
        #endregion

        #endregion

        #region AddCustomData

        #region positive
        /// <summary>
        /// This method is used to test the add custom data.
        /// </summary>
        [TestMethod]
        public void Test_AddCustomData()
        {
            using (var client = GetClient())
            {
                var userdetails = UserDetailsClientTest.GetUserById(UserId);
                var data = DataViewsClientUnitTest.GetDataViewDetails(UserDetailsFormEntityId);
                Assert.IsNotNull(data);
                List<FieldDataDTO> fieldDataDTOList = new List<FieldDataDTO>();
                foreach (var field in data.fields)
                {
                    if (field.is_extended_field)
                    {
                        var fieldDataDTO = new FieldDataDTO()
                        {
                            field_id = field.field_id,

                        };
                        if (field.field_type.Equals("Date"))
                        {
                            fieldDataDTO.value = DateTime.Now.ToString();
                        }
                        else if (field.field_metadata.type_id.Equals(UserId.ToString()))
                        {
                            fieldDataDTO.value = "1";
                        }
                        else
                        {
                            fieldDataDTO.value = "test";
                        }
                        fieldDataDTOList.Add(fieldDataDTO);
                    }

                }

                var customddata = new CustomDataDTO()
                {
                    entity_id = data.entity_id,
                    tenant_id = data.tenant_id,
                    ref_id = userdetails.id.ToString(),
                    data = fieldDataDTOList.ToArray()
                };
                client.AddCustomData(customddata);
            }
        }
        #endregion
        #endregion AddCustomData

        #region Test_GetCustomData

        #region positive
        /// <summary>
        /// This method is used to test the get custom data.
        /// </summary>
        [TestMethod]
        public void Test_GetCustomData()
        {
            using (var client = GetClient())
            {
                var userdetails = UserDetailsClientTest.GetUserById(UserId);
                var data = client.GetCustomData(userdetails.id.ToString(), UserEntityId);
                Assert.IsNotNull(data);
            }
        }

        #endregion
        #endregion Test_GetCustomData

        #region Test_DeleteCustomData

        #region positive

        /// <summary>
        /// This method is used to test the delete custom data.
        /// </summary>
        [TestMethod]
        public void Test_DeleteCustomData()
        {
            using (var client = GetClient())
            {
                var userdetails = UserDetailsClientTest.GetUserById(UserId);
                client.DeleteCustomData(userdetails.id.ToString(), UserEntityId);
            }
        }

        #endregion
        #endregion Test_DeleteCustomData

        /// <summary>
        /// Client.
        /// </summary>
        /// <returns></returns>
        private static EntityMetadatasClient GetClient()
        {
            var client = new EntityMetadatasClient(
               baseAddress: EntityMetadatasClientBaseAddress,
               username: ClientUserName,
               api_key: APIKey);
            return client;
        }
    }

}