﻿using System;
using CelloSaaS.RestApiSDK;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WebApiSDKUnitTest
{
    /// <summary>
    /// This class is resposible for unit testing of MeteringClient API.
    /// </summary>
    [TestClass]
    public class MeteringClientUnitTest
    {
        public static string RootURI = System.Configuration.ConfigurationSettings.AppSettings["RootURI"].ToString();
        public static string APIEndPoint = "Metering/";
        public static string MeteringClientBaseAddress = RootURI + APIEndPoint;

        public static string APIKey = System.Configuration.ConfigurationSettings.AppSettings["APIKey"].ToString();
        public static string ClientUserName = System.Configuration.ConfigurationSettings.AppSettings["ClientUserName"].ToString();

        public static string ProjectUsageCode = "ProjectUsage";
        public static string UserUsageCode = "User";

        public static string InvalidUsageCode = "Systems";

        ExceptionHelper exceptionHelper = null;

        public MeteringClientUnitTest()
        {
            exceptionHelper = new ExceptionHelper();
        }
        /// <summary>
        /// This method is used to test the get metering usage.
        /// </summary>
        [TestMethod]
        public void TestGetMeteringUsage()
        {
            using (var client = GetClient())
            {
                TestIncrement();
                var amt = client.GetMeterAmount(UserUsageCode);
                Assert.AreNotEqual(amt, 0.0);
                TestDecrement();
            }
        }
        /// <summary>
        /// This method is used to test the get metering usage with invalid usage code.
        /// </summary>
        [TestMethod]
        public void TestGetMeteringUsage_InvalidUsageCode()
        {
            using (var client = GetClient())
            {
                var amt = client.GetMeterAmount(InvalidUsageCode);
                Assert.AreEqual(amt, 0.0);
            }
        }
        /// <summary>
        /// This method is used to test the get metering usage with empty usage code.
        /// </summary>
        [TestMethod]
        public void TestGetMeteringUsage_EmptyUsageCode()
        {
            try
            {
                using (var client = GetClient())
                {
                    var amt = client.GetMeterAmount("");
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the increment of metering.
        /// </summary>
        [TestMethod]
        public void TestIncrement()
        {
            using (var client = GetClient())
            {
                var amt = client.GetMeterAmount(UserUsageCode);
                client.Increment(UserUsageCode, 1.0);
                var actual = client.GetMeterAmount(UserUsageCode);
                Assert.AreEqual(amt + 1.0, actual);
            }
        }       
        /// <summary>
        /// This method is used to test the increment with beyond limit.
        /// </summary>
        [TestMethod]
        public void TestIncrement_BeyondLimit()
        {
            try
            {
                using (var client = GetClient())
                {
                    var amt = client.GetMeterAmount(UserUsageCode);
                    client.Increment(UserUsageCode, 10000000000000000);
                    var actual = client.GetMeterAmount(UserUsageCode);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.ValidationErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the increment with floating point and invalid usage code.
        /// </summary>
        [TestMethod]
        public void TestIncrement_FloatingPointWithInvalidUsageCode()
        {
            try
            {
                using (var client = GetClient())
                {
                    client.Increment(InvalidUsageCode, 2.3);
                    var actual = client.GetMeterAmount(InvalidUsageCode);
                }
            }
            catch (Exception Ex)
            {
                string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                Assert.IsNotNull(errorCode);
                Assert.AreEqual(errorCode, ExceptionHelper.LicenseExceptionErrorCode);
            }
        }
        /// <summary>
        /// This method is used to test the increment without decimal point.
        /// </summary>
        [TestMethod]
        public void TestIncrement_WithoutDecimalPoint()
        {
            using (var client = GetClient())
            {
                var amt = client.GetMeterAmount(UserUsageCode);
                client.Increment(UserUsageCode, 1);
                var actual = client.GetMeterAmount(UserUsageCode);
                Assert.AreEqual(amt + 1.0, actual);
            }
        }
        /// <summary>
        /// This method is used to test the decrement of metering.
        /// </summary>
        [TestMethod]
        public void TestDecrement()
        {
            using (var client = GetClient())
            {
                var amt = client.GetMeterAmount(UserUsageCode);
                client.Decrement(UserUsageCode, 1.0);
                var actual = client.GetMeterAmount(UserUsageCode);
                Assert.AreEqual(amt - 1.0, actual);
            }
        }
        /// <summary>
        /// This method is used to test the decrement without decimal point.
        /// </summary>
        [TestMethod]
        public void TestDecrement_WithoutDecimalPoint()
        {
            using (var client = GetClient())
            {
                var amt = client.GetMeterAmount(UserUsageCode);
                client.Decrement(UserUsageCode, 1);
                var actual = client.GetMeterAmount(UserUsageCode);
                Assert.AreEqual(amt - 1.0, actual);
            }
        }
        /// <summary>
        /// This method is used to test the decrement with floating point and invalid usage code.
        /// </summary>
        [TestMethod]
        public void TestDecrement_FloatingPointInvalidUsageCode()
        {
                using (var client = GetClient())
                {
                    client.Decrement(InvalidUsageCode, 2.3);
                    var actual = client.GetMeterAmount(InvalidUsageCode);
                    Assert.AreEqual(actual, 0.0);
                }

        }
        /// <summary>
        /// This method is used to test the search the metering.
        /// </summary>
        [TestMethod]
        public void TestSearch()
        {
            using (var client = GetClient())
            {
                var res = client.Search(new MeteringSearchConditionDTO
                {
                    usage_code = UserUsageCode,
                    end_date = DateTime.Now
                });

                Assert.IsNotNull(res);
                Assert.IsNotNull(res.result);
                Assert.AreNotEqual(res.total_count, 0);
            }
        }
        /// <summary>
        /// This method is used to test the search with invalid date.
        /// </summary>
        [TestMethod]
        public void TestSearch_Outdated()
        {
            using (var client = GetClient())
            {
                DateTime date = new DateTime(1990, 05, 06);
                var results = client.Search(new MeteringSearchConditionDTO
                {

                    usage_code =UserUsageCode,
                    end_date = date
                });
                Assert.IsNull(results.result);
                Assert.AreEqual(results.total_count, 0);            
            }
        }
        /// <summary>
        /// This method is used to test the search with empty usage code.
        /// </summary>
        [TestMethod]
        public void TestSearch_EmpyUsageCode()
        {
            using (var client = GetClient())
            {
                var res = client.Search(new MeteringSearchConditionDTO
                {
                    usage_code = "",
                    end_date = DateTime.Now
                });
                Assert.IsNull(res.result);
            }
        }
        /// <summary>
        /// This method is used to test the search without end date.
        /// </summary>
        [TestMethod]
        public void TestSearch_WithoutEndDate()
        {
            using (var client = GetClient())
            {
                var res = client.Search(new MeteringSearchConditionDTO
                {
                    usage_code = UserUsageCode
                });
                Assert.IsNotNull(res.result);
            }
        }
        /// <summary>
        /// This method is used to test the search usage.
        /// </summary>
        [TestMethod]
        public void TestSearchUsage()
        {
            using (var client = GetClient())
            {
                var res = client.SearchUsage(new UsageSearchConditionDTO
                {
                    usage_codes = new string[] { UserUsageCode },
                });

                Assert.IsNotNull(res);
                Assert.IsNotNull(res.result);
                Assert.AreNotEqual(res.total_count, 0);
            }
        }
        /// <summary>
        /// This method is used to test the search usage with invalid usage code.
        /// </summary>
        [TestMethod]
        public void TestSearchUsage_InvalidUsageCode()
        {
            using (var client = GetClient())
            {
                try
                {
                    var res = client.SearchUsage(new UsageSearchConditionDTO
                    {
                        usage_codes = new string[] { InvalidUsageCode },
                    });
                }
                catch (Exception Ex)
                {
                    string errorCode = exceptionHelper.GetErrorCode(Ex.Message);
                    Assert.IsNotNull(errorCode);
                    Assert.AreEqual(errorCode, ExceptionHelper.ArgumentExceptionErrorCode);
                }
            }
        }
        private static MeteringClient GetClient()
        {
            var client = new MeteringClient(
              baseAddress: MeteringClientBaseAddress,
              username: ClientUserName,
              api_key: APIKey);
            return client;
        }
    }
}
